@@ -1,6 +1,7 @@
 from .logger import logger
 from PySimultan import TemplateParser, DataModel, Template, yaml
 from .geometry.extended_geometry import ExtendedVertex, ExtendedEdge, ExtendedEdgeLoop, ExtendedFace, ExtendedVolume
+from .geometry.extended_default_types import ExtendedBuildInFace
 from .foi import ReferenceList
 from .weather import Weather
 
@@ -12,3 +13,5 @@ TemplateParser.geo_bases['volume'] = ExtendedVolume
 
 TemplateParser.bases['ReferenceList'] = ReferenceList
 TemplateParser.bases['Weather'] = Weather
+TemplateParser.bases['Geometric Area'] = ExtendedBuildInFace
+TemplateParser.bases['Geometrische_Flächen'] = ExtendedBuildInFace
