@@ -36,7 +36,6 @@ def mesh_planar_face(face, mesh_size=0.1):
 
         mesh = geom.generate_mesh(dim=2, verbose=False)
 
-        # mesh.write('face_with_opening.vtk')
     return mesh
 
 
@@ -186,7 +185,7 @@ def generate_surface_mesh(*args, **kwargs):
     max_size = kwargs.get('max_size', 10)
     verbose = kwargs.get('verbose', False)
     mesh_size_from_faces = kwargs.get('mesh_size_from_faces', True)
-    method = kwargs.get('method', '')
+    method = kwargs.get('method', 'robust')
 
     logger.info((f'Generating surface mesh:\n'
                  f'Number Vertices: {vertices.__len__()}\n'
@@ -204,6 +203,7 @@ def generate_surface_mesh(*args, **kwargs):
             polys = {}
 
             for face in faces:
+
                 holes = []
 
                 if mesh_size_from_faces:
