@@ -5,6 +5,7 @@ import time
 import traceback
 from multiprocessing import Pool, cpu_count
 from scipy import integrate
+import dill as pickle
 
 import meshio
 import numpy as np
@@ -14,7 +15,7 @@ import sqlalchemy
 import trimesh
 from PySimultan import logger as py_sim_logger
 from meshio import Mesh as MioMesh
-from pandas.io import sql
+# from pandas.io import sql
 from psycopg2.extensions import register_adapter, AsIs
 from psycopg2.extras import Json
 from sqlalchemy import Table, Column, MetaData
@@ -25,8 +26,10 @@ from trimesh import Trimesh
 
 from . import TemplateParser, Template, yaml, DataModel
 from . import logger
-from .client.client import Client, next_free_port
-from .docker.docker_manager import ShadingService, DatabaseService
+from .client.client import ShadingClient
+from .client.client import ViewFactorClient
+from .client.client import next_free_port
+from .docker.docker_manager import ShadingService, DatabaseService, ViewFactorService
 from .geometry.scene import Scene
 from .geometry.utils import calc_aoi
 from .gui.dialogs import askComboValue
@@ -35,8 +38,7 @@ from .radiation.utils import create_sun_window
 from .utils import df_interpolate_at, write_face_results
 from .db_utils import DBInterface
 
-from sqlalchemy.pool import QueuePool
-
+# from sqlalchemy.pool import QueuePool
 
 try:
     import importlib.resources as pkg_resources
@@ -238,9 +240,10 @@ class ProjectLoader(object):
         self.typed_data = None
 
         self.setup_components = []
-
         self.app = kwargs.get('app', None)
 
+        self.loaded = False
+
     def load_project(self):
 
         logger.setLevel('INFO')
@@ -260,10 +263,12 @@ class ProjectLoader(object):
         self.setup_components = [x for x in self.template_parser.template_classes['ShadingAnalysis']._cls_instances if bool(x.Active)]
         logger.info(f'Found {self.setup_components.__len__()} active Shading Analyses')
 
-        print('done')
+        self.loaded = True
 
     def run(self, *args, **kwargs):
 
+        self.loaded = True
+
         for setup_component in self.setup_components:
 
             try:
@@ -295,6 +300,15 @@ class ProjectLoader(object):
 
         logger.info('Finished all active analyses')
 
+    def close_project(self):
+        if self.loaded:
+            self.typed_data = None
+            self.setup_components = None
+            self.loaded = False
+            if self.data_model is not None:
+                self.data_model.cleanup()
+            self.data_model = None
+
 
 class ShadingAnalysis(object):
 
@@ -306,6 +320,7 @@ class ShadingAnalysis(object):
         self._mesh = None
         self._foi_mesh = None
         self._hull_mesh = None
+        self._view_factor_mesh = None
         self._dti = None
 
         self.user_name = kwargs.get('user_name', 'admin')
@@ -321,6 +336,7 @@ class ShadingAnalysis(object):
         self.setup_component = kwargs.get('setup_component', None)
 
         self._shading_service = None
+        self._view_factor_service = None
         self._db_service = None
 
         self._db_interface = None
@@ -404,6 +420,34 @@ class ShadingAnalysis(object):
     def shading_service(self, value):
         self._shading_service = value
 
+    @property
+    def view_factor_service(self):
+        if self._view_factor_service is None:
+            serv_work_workdir = os.path.join(self.setup_component.run_configuration.ExportDirectory,
+                                             'serv_work_workdir')
+            port = next_free_port(port=10006, max_port=65535)
+
+            ftp_port_1 = next_free_port(port=8007, max_port=65535)
+            ftp_port_2 = next_free_port(port=ftp_port_1 + 1, max_port=65535)
+
+            if not os.path.isdir(serv_work_workdir):
+                os.makedirs(serv_work_workdir, exist_ok=True)
+
+            self._view_factor_service = ViewFactorService(workdir=serv_work_workdir,
+                                                          port=port,
+                                                          num_workers=int(self.setup_component.run_configuration.NumWorkers),
+                                                          logging_mode=self.setup_component.run_configuration.LogLevel,
+                                                          ftp_user=self.user_name,
+                                                          ftp_password=self.password,
+                                                          ftp_port_1=ftp_port_1,
+                                                          ftp_port_2=ftp_port_2,
+                                                          ftp_passive_ports='11000-11050')
+        return self._view_factor_service
+
+    @view_factor_service.setter
+    def view_factor_service(self, value):
+        self._view_factor_service = value
+
     @property
     def id(self):
         if self.setup_component is None:
@@ -526,6 +570,13 @@ class ShadingAnalysis(object):
     def db_name(self):
         return str(self.id.GlobalId)
 
+    @property
+    def view_factor_mesh(self):
+        # for view_factor_calculation only coarse mesh is neccessary
+        if self._view_factor_mesh is None:
+            self._view_factor_mesh = self.generate_view_factor_mesh()
+        return self._view_factor_mesh
+
     def select_geo_model(self):
 
         if self.setup_component.geometry.geometry._wrapped_obj.ReferencedAssets.Items.Items.__len__() == 0:
@@ -595,6 +646,9 @@ class ShadingAnalysis(object):
                                    faces=self.mesh.cells_dict['triangle'][np.where(self.mesh.cell_data['foi'][0]), :][0])
         return foi_mesh
 
+    def generate_view_factor_mesh(self):
+        pass
+
     def update_fois(self):
 
         if self.foi_faces is None:
@@ -610,9 +664,12 @@ class ShadingAnalysis(object):
     def run(self):
         self.db_service.keep_running = True
         with self.db_service:
-            if self.setup_component.run_configuration.RunShadingFactorCalculation:
+            if bool(self.setup_component.run_configuration.RunShadingFactorCalculation):
                 self.run_shading_analysis()
                 self.export_results()
+            if bool(self.setup_component.run_configuration.RunViewFactorCalculation):
+                self.run_view_factor_analysis()
+                self.export_view_factor_results()
         self.db_service.shut_down_db_service()
 
     def run_shading_analysis(self):
@@ -656,7 +713,7 @@ class ShadingAnalysis(object):
 
             # num_cells = self.mesh.cells_dict['triangle'].shape[0]
 
-            my_client = Client(ip=f'tcp://localhost:{self.shading_service.port}')
+            my_client = ShadingClient(ip=f'tcp://localhost:{self.shading_service.port}')
 
             # my_shading_service.write_compose_file('docker_compose_test.yml')
 
@@ -735,6 +792,33 @@ class ShadingAnalysis(object):
 
             self.evaluate_shading_results()
 
+    def run_view_factor_analysis(self):
+        logger.info(f'Starting view factor analysis {self.id}...')
+
+        with self.db_service:
+            my_client = ViewFactorClient(ip=f'tcp://localhost:{self.view_factor_service.port}')
+            self.update_fois()
+
+            if self.foi_faces is not None:
+                faces = self.foi_faces
+            else:
+                faces = set(self.geo_model.faces)
+
+            logger.info(f'Starting shading service...')
+            with self.view_factor_service:
+                logger.info(f'Sending view factor mesh to clients')
+                my_client.send_mesh(self.view_factor_mesh)
+
+                for face in faces:
+                    try:
+                        f_dump = pickle.dumps(face)
+                    except Exception as e:
+                        logger.error(
+                            f'Error writing dni to database:\n{e}\n{traceback.format_exc()}\n{sys.exc_info()[2]}')
+                    print('pickled_face')
+                    # my_client.calc_view_factor(face=)
+                print('done')
+
     def write_mesh_to_db(self, mesh):
 
         logger.info(f'Writing mesh to database...')
@@ -1079,6 +1163,9 @@ class ShadingAnalysis(object):
                                            workbook,
                                            self.geo_model.FaceCls)
 
+    def export_view_factor_results(self):
+        pass
+
     def generate_hull_mesh(self):
 
         hull_mesh = MioMesh(points=self.mesh.points,
@@ -1210,7 +1297,7 @@ class RTEngine(object):
         df_row = args[0][1]
 
         if self.client is None:
-            self.client = Client(ip=f'tcp://localhost:{self.port}')
+            self.client = ShadingClient(ip=f'tcp://localhost:{self.port}')
 
         irradiation_vector = df_row['irradiation_vector']
         sun_window = df_row['windows']
@@ -1251,7 +1338,7 @@ def calc_timestep_async(port=None,
                         process=None):
 
     # start_time = time.time()
-    client = Client(ip=f'tcp://localhost:{port}')
+    client = ShadingClient(ip=f'tcp://localhost:{port}')
 
     # logger.info(f'processing timestep: {date}')
 
